<?php

// autoload_static.php @generated by Composer

namespace Composer\Autoload;

class ComposerStaticInit61236a40bbfb35b64aa87a1e1e03e69d
{
    public static $files = array (
        'a2c78434f64e5f5ed402f42eee19c025' => __DIR__ . '/..' . '/ipl/stdlib/src/functions_include.php',
        '6076de347104821999fcfc82c8f19bc5' => __DIR__ . '/..' . '/ipl/i18n/src/functions_include.php',
        '9d2b9fc6db0f153a0a149fefb182415e' => __DIR__ . '/..' . '/symfony/polyfill-php84/bootstrap.php',
        '7b11c4dc42b3b3023073cb14e519683c' => __DIR__ . '/..' . '/ralouphie/getallheaders/src/getallheaders.php',
        '20b38f856de532d80624e561e6db2dee' => __DIR__ . '/..' . '/ipl/sql/src/pdo-driver-consts-forward-compat.php',
        'e39a8b23c42d4e1452234d762b03835a' => __DIR__ . '/..' . '/ramsey/uuid/src/functions.php',
        'ad155f8f1cf0d418fe49e248db8c661b' => __DIR__ . '/..' . '/react/promise/src/functions_include.php',
        '662a729f963d39afe703c9d9b7ab4a8c' => __DIR__ . '/..' . '/symfony/polyfill-php83/bootstrap.php',
        '8e4ccce73649a2b516ec3b4571432da5' => __DIR__ . '/..' . '/ipl/scheduler/src/register_cron_aliases.php',
    );

    public static $prefixLengthsPsr4 = array (
        'i' =>
        array (
            'ipl\\Web\\' => 8,
            'ipl\\Validator\\' => 14,
            'ipl\\Stdlib\\' => 11,
            'ipl\\Sql\\' => 8,
            'ipl\\Scheduler\\' => 14,
            'ipl\\Orm\\' => 8,
            'ipl\\I18n\\' => 9,
            'ipl\\Html\\' => 9,
        ),
        'S' =>
        array (
            'Symfony\\Polyfill\\Php84\\' => 23,
            'Symfony\\Polyfill\\Php83\\' => 23,
        ),
        'R' =>
        array (
            'Recurr\\' => 7,
            'React\\Promise\\' => 14,
            'React\\EventLoop\\' => 16,
            'Ramsey\\Uuid\\' => 12,
            'Ramsey\\Collection\\' => 18,
        ),
        'P' =>
        array (
            'Psr\\Log\\' => 8,
            'Psr\\Http\\Message\\' => 17,
        ),
        'G' =>
        array (
            'GuzzleHttp\\Psr7\\' => 16,
        ),
        'E' =>
        array (
            'Evenement\\' => 10,
        ),
        'D' =>
        array (
            'Doctrine\\Deprecations\\' => 22,
            'Doctrine\\Common\\Collections\\' => 28,
        ),
        'C' =>
        array (
            'Cron\\' => 5,
        ),
        'B' =>
        array (
            'Brick\\Math\\' => 11,
        ),
    );

    public static $prefixDirsPsr4 = array (
        'ipl\\Web\\' =>
        array (
            0 => __DIR__ . '/..' . '/ipl/web/src',
        ),
        'ipl\\Validator\\' =>
        array (
            0 => __DIR__ . '/..' . '/ipl/validator/src',
        ),
        'ipl\\Stdlib\\' =>
        array (
            0 => __DIR__ . '/..' . '/ipl/stdlib/src',
        ),
        'ipl\\Sql\\' =>
        array (
            0 => __DIR__ . '/..' . '/ipl/sql/src',
        ),
        'ipl\\Scheduler\\' =>
        array (
            0 => __DIR__ . '/..' . '/ipl/scheduler/src',
        ),
        'ipl\\Orm\\' =>
        array (
            0 => __DIR__ . '/..' . '/ipl/orm/src',
        ),
        'ipl\\I18n\\' =>
        array (
            0 => __DIR__ . '/..' . '/ipl/i18n/src',
        ),
        'ipl\\Html\\' =>
        array (
            0 => __DIR__ . '/..' . '/ipl/html/src',
        ),
        'Symfony\\Polyfill\\Php84\\' =>
        array (
            0 => __DIR__ . '/..' . '/symfony/polyfill-php84',
        ),
        'Symfony\\Polyfill\\Php83\\' =>
        array (
            0 => __DIR__ . '/..' . '/symfony/polyfill-php83',
        ),
        'Recurr\\' =>
        array (
            0 => __DIR__ . '/..' . '/simshaun/recurr/src/Recurr',
        ),
        'React\\Promise\\' =>
        array (
            0 => __DIR__ . '/..' . '/react/promise/src',
        ),
        'React\\EventLoop\\' =>
        array (
            0 => __DIR__ . '/..' . '/react/event-loop/src',
        ),
        'Ramsey\\Uuid\\' =>
        array (
            0 => __DIR__ . '/..' . '/ramsey/uuid/src',
        ),
        'Ramsey\\Collection\\' =>
        array (
            0 => __DIR__ . '/..' . '/ramsey/collection/src',
        ),
        'Psr\\Log\\' =>
        array (
            0 => __DIR__ . '/..' . '/psr/log/Psr/Log',
        ),
        'Psr\\Http\\Message\\' =>
        array (
            0 => __DIR__ . '/..' . '/psr/http-factory/src',
            1 => __DIR__ . '/..' . '/psr/http-message/src',
        ),
        'GuzzleHttp\\Psr7\\' =>
        array (
            0 => __DIR__ . '/..' . '/guzzlehttp/psr7/src',
        ),
        'Evenement\\' =>
        array (
            0 => __DIR__ . '/..' . '/evenement/evenement/src',
        ),
        'Doctrine\\Deprecations\\' =>
        array (
            0 => __DIR__ . '/..' . '/doctrine/deprecations/src',
        ),
        'Doctrine\\Common\\Collections\\' =>
        array (
            0 => __DIR__ . '/..' . '/doctrine/collections/src',
        ),
        'Cron\\' =>
        array (
            0 => __DIR__ . '/..' . '/dragonmantank/cron-expression/src/Cron',
        ),
        'Brick\\Math\\' =>
        array (
            0 => __DIR__ . '/..' . '/brick/math/src',
        ),
    );

    public static $prefixesPsr0 = array (
        'Z' =>
        array (
            'Zend_' =>
            array (
                0 => __DIR__ . '/..' . '/icinga/zf1/library',
            ),
        ),
        'L' =>
        array (
            'Less' =>
            array (
                0 => __DIR__ . '/..' . '/wikimedia/less.php/lib',
            ),
        ),
        'A' =>
        array (
            'AssetLoader' =>
            array (
                0 => __DIR__ . '/../..' . '/',
            ),
        ),
    );

    public static $classMap = array (
        'Composer\\InstalledVersions' => __DIR__ . '/..' . '/composer/InstalledVersions.php',
        'DateError' => __DIR__ . '/..' . '/symfony/polyfill-php83/Resources/stubs/DateError.php',
        'DateException' => __DIR__ . '/..' . '/symfony/polyfill-php83/Resources/stubs/DateException.php',
        'DateInvalidOperationException' => __DIR__ . '/..' . '/symfony/polyfill-php83/Resources/stubs/DateInvalidOperationException.php',
        'DateInvalidTimeZoneException' => __DIR__ . '/..' . '/symfony/polyfill-php83/Resources/stubs/DateInvalidTimeZoneException.php',
        'DateMalformedIntervalStringException' => __DIR__ . '/..' . '/symfony/polyfill-php83/Resources/stubs/DateMalformedIntervalStringException.php',
        'DateMalformedPeriodStringException' => __DIR__ . '/..' . '/symfony/polyfill-php83/Resources/stubs/DateMalformedPeriodStringException.php',
        'DateMalformedStringException' => __DIR__ . '/..' . '/symfony/polyfill-php83/Resources/stubs/DateMalformedStringException.php',
        'DateObjectError' => __DIR__ . '/..' . '/symfony/polyfill-php83/Resources/stubs/DateObjectError.php',
        'DateRangeError' => __DIR__ . '/..' . '/symfony/polyfill-php83/Resources/stubs/DateRangeError.php',
        'Deprecated' => __DIR__ . '/..' . '/symfony/polyfill-php84/Resources/stubs/Deprecated.php',
        'Override' => __DIR__ . '/..' . '/symfony/polyfill-php83/Resources/stubs/Override.php',
        'Pdo\\Dblib' => __DIR__ . '/..' . '/symfony/polyfill-php84/Resources/stubs/Pdo/Dblib.php',
        'Pdo\\Firebird' => __DIR__ . '/..' . '/symfony/polyfill-php84/Resources/stubs/Pdo/Firebird.php',
        'Pdo\\Mysql' => __DIR__ . '/..' . '/symfony/polyfill-php84/Resources/stubs/Pdo/Mysql.php',
        'Pdo\\Odbc' => __DIR__ . '/..' . '/symfony/polyfill-php84/Resources/stubs/Pdo/Odbc.php',
        'Pdo\\Pgsql' => __DIR__ . '/..' . '/symfony/polyfill-php84/Resources/stubs/Pdo/Pgsql.php',
        'Pdo\\Sqlite' => __DIR__ . '/..' . '/symfony/polyfill-php84/Resources/stubs/Pdo/Sqlite.php',
        'ReflectionConstant' => __DIR__ . '/..' . '/symfony/polyfill-php84/Resources/stubs/ReflectionConstant.php',
        'RoundingMode' => __DIR__ . '/..' . '/symfony/polyfill-php84/Resources/stubs/RoundingMode.php',
        'SQLite3Exception' => __DIR__ . '/..' . '/symfony/polyfill-php83/Resources/stubs/SQLite3Exception.php',
        'lessc' => __DIR__ . '/..' . '/wikimedia/less.php/lessc.inc.php',
    );

    public static function getInitializer(ClassLoader $loader)
    {
        return \Closure::bind(function () use ($loader) {
            $loader->prefixLengthsPsr4 = ComposerStaticInit61236a40bbfb35b64aa87a1e1e03e69d::$prefixLengthsPsr4;
            $loader->prefixDirsPsr4 = ComposerStaticInit61236a40bbfb35b64aa87a1e1e03e69d::$prefixDirsPsr4;
            $loader->prefixesPsr0 = ComposerStaticInit61236a40bbfb35b64aa87a1e1e03e69d::$prefixesPsr0;
            $loader->classMap = ComposerStaticInit61236a40bbfb35b64aa87a1e1e03e69d::$classMap;

        }, null, ClassLoader::class);
    }
}
