<?php

namespace ipl\Web\FormElement\TermInput;

use ipl\Web\FormElement\SearchSuggestions;

/**
 * @deprecated Use {@see SearchSuggestions} instead
 */
class TermSuggestions extends SearchSuggestions
{
}
