i.icon {
  vertical-align: middle; // Firefox will place icons weird otherwise
  display: inline-block;

  font-style: normal;
  line-height: 1;

  &::before {
    display: block;
    min-width: 1em;
    margin-right: .2em;

    text-align: center;
    text-decoration: inherit;
  }
}
