:root, :host {
  --fa-font-regular: normal 400 1em/1 "Font Awesome 6 Free";
  --fa-font-solid: normal 900 1em/1 "Font Awesome 6 Free";
}

@font-face {
  font-family: 'Font Awesome 6 Free';
  font-style: normal;
  font-weight: 400;
  font-display: block;
  src: url('@{iplWebAssets}/font/awesome/fa-regular-400.woff2') format('woff2'),
       url('@{iplWebAssets}/font/awesome/fa-regular-400.ttf') format('truetype');
}

.far,
.fa-regular {
  font-family: 'Font Awesome 6 Free';
  font-weight: 400;
}

@font-face {
  font-family: 'Font Awesome 6 Free';
  font-style: normal;
  font-weight: 900;
  font-display: block;
  src: url('@{iplWebAssets}/font/awesome/fa-solid-900.woff2') format('woff2'),
       url('@{iplWebAssets}/font/awesome/fa-solid-900.ttf') format('truetype');
}

.fa,
.fas,
.fa-solid {
  font-family: 'Font Awesome 6 Free';
  font-weight: 900;
}
