<?php

// autoload_namespaces.php @generated by Composer

$vendorDir = dirname(__DIR__);
$baseDir = dirname($vendorDir);

return array(
    'Zend_' => array($vendorDir . '/icinga/zf1/library'),
    'Less' => array($vendorDir . '/wikimedia/less.php/lib'),
    'AssetLoader' => array($baseDir . '/'),
);
